// import { callUnomi, resolvePromise, createPromise } from "../utils/connection";
import { callUnomi } from "../utils/connection";
import { validateSegmentObject, reformatSegment } from "../utils/validation";
import { FilteredResponse } from "../../src/types/sdkResponse";
import { UsedProperties } from "../../src/types/segments";

async function resolveReformatSegment(url: string, headers: Record<string, string>): FilteredResponse {
    const callObjectPromise = await callUnomi("GET", url, null, headers, 200);

    if (callObjectPromise.statusCode === 200) {
        return new Promise((resolve, reject) => {
            resolve(reformatSegment(callObjectPromise, callObjectPromise.responseData));
        });
    }

    else {
        return new Promise((resolve, reject) => {
            resolve(callObjectPromise);
        });
    }
}

/**
 * @function getAll
 * @param {string} baseUrl 
 * @param {Record<string, string>} headers 
 * @param {string} sortOrder 
 * @param {number} limit 
 * @param {number} offset 
 * @returns {FilteredResponse}
 */
export function getAll(baseUrl: string, headers: Record<string, string>, sortOrder: string, limit: number, offset: number): FilteredResponse {
    const url = baseUrl + `/cxs/segments?size=${limit}&offset=${offset}&sort=${sortOrder}`;

    return callUnomi("GET", url, null, headers, 200);
}

/**
 * @function get
 * @param {string} baseUrl 
 * @param {Record<string, string>} headers 
 * @param {string} segmentId 
 * @returns {FilteredResponse}
 */
export function get(baseUrl: string, headers: Record<string, string>, segmentId: string): FilteredResponse {
    const url = baseUrl + `/cxs/segments/${segmentId}`;

    return resolveReformatSegment(url, headers);
}

/**
 * @function create
 * @param {string} baseUrl 
 * @param {Record<string, string>} headers 
 * @param {object} properties 
 * @returns {FilteredResponse}
 */
export function create(baseUrl: string, headers: Record<string, string>, params: UsedProperties): FilteredResponse {
    const url = baseUrl + `/cxs/segments`;

    const validatedSegmentObject = validateSegmentObject(params, false);

    if (validatedSegmentObject.areSubConditionsValid) {
        return callUnomi("POST", url, validatedSegmentObject.segmentObject, headers, 204);
    }

    else {
        return callUnomi("POST", url, validatedSegmentObject.segmentObject, headers, 204, validatedSegmentObject.errorCode);
    }
}

/**
 * @function update
 * @param {string} baseUrl 
 * @param {Record<string, string>} headers 
 * @param {object} properties 
 * @returns {FilteredResponse}
 */
export function update(baseUrl: string, headers: Record<string, string>, params: UsedProperties): FilteredResponse {
    const url = baseUrl + `/cxs/segments`;

    const validatedSegmentObject = validateSegmentObject(params, true);

    if (validatedSegmentObject.areSubConditionsValid) {
        return callUnomi("POST", url, validatedSegmentObject.segmentObject, headers, 204);
    }

    else {
        return callUnomi("POST", url, validatedSegmentObject.segmentObject, headers, 204, validatedSegmentObject.errorCode);
    }
}

/**
 * @function delete
 * @param {string} baseUrl 
 * @param {Record<string, string>} headers 
 * @param {string} segmentId 
 * @returns {FilteredResponse}
 */
export function deleteSegment(baseUrl: string, headers: Record<string, string>, segmentId: string): FilteredResponse {
    const url = baseUrl + `/cxs/segments/${segmentId}`;

    return callUnomi("DELETE", url, null, headers, 200);
}