import { FilteredResponse, sdkResponse, sdkCall } from "../../src/types/sdkResponse";

/**
 * @function callUnomi
 * @param {string} method 
 * @param {string} url 
 * @param {object} body 
 * @param {Record<string, string>} headers 
 * @param {string} successStatus 
 * @returns {FilteredResponse}
 */
export async function callUnomi(method: string, url: string, body: object, headers: Record<string, string>, successStatus: number, errorCode?: string): FilteredResponse {

    try {

        if (errorCode) { // if errorCode is not empty

            return {
                statusCode: 500,
                errorMessage: "Something went wrong processing this request. Please try again later or contact support if this error persists. Error Code: " + errorCode,
                response: {
                    method: method,
                    url: url,
                    body: body,
                    successStatus: successStatus
                }
            };
        }

        let response;

        if (body === null) {
            console.log(headers);
            response = await fetch(url, {
                method: method,
                headers: headers,
                credentials: "omit",
                mode: 'cors',
            });
        }

        else {

            response = await fetch(url, {
                method: method,
                headers: headers,
                body: JSON.stringify(body),
                credentials: "omit",
                mode: 'cors',
            });
        }

        let data = new Object;

        if (response.status != 204) {

            data = await response.json();
        }

        if (response.status === successStatus) {
            return {
                statusCode: response.status,
                responseData: data
            };
        }

        return {
            statusCode: response.status,
            errorMessage: "Something went wrong processing this request. Please try again later or contact support if this error persists. Error Code: 5xU04",
            response: response
        };

    } catch (err) {

        return {
            statusCode: 500,
            errorMessage: "Something went wrong processing this request. Please try again later or contact support if this error persists. Error Code: 5xU01",
            response: {
                method: method,
                url: url,
                body: body,
                successStatus: successStatus
            }
        };
    }
}

/**
 * @function callElasticsearch
 * @param {string} method 
 * @param {string} url 
 * @param {object} body 
 * @param {string} headers 
 * @param {string} successStatus 
 * @returns {FilteredResponse}
 */
export async function callElasticsearch(method: string, url: string, headers: Record<string, string>, successStatus: number): FilteredResponse {

    try {

        let response;

        response = await fetch(url, {
            method: method,
            headers: headers,
            credentials: "omit",
            mode: 'cors',
        });

        let data = await response.json();

        data = data["context-profile"]["mappings"]["properties"]["properties"]["properties"];

        if (response.status === successStatus) {

            return {
                statusCode: response.status,
                responseData: Object.keys(data)   // list of the keys of the data object
            };
        }

        return {
            statusCode: response.status,
            errorMessage: "Something went wrong processing this request. Please try again later or contact support if this error persists. Error Code: 5xU05",
            response: response
        };

    } catch (err) {

        return {
            statusCode: 500,
            errorMessage: "Something went wrong processing this request. Please try again later or contact support if this error persists. Error Code: 5xU06",
            response: {
                method: method,
                url: url,
                successStatus: successStatus
            }
        };
    }
}

/**
 * @function callResponse
 * @param {string} method 
 * @param {string} url 
 * @param {any} body 
 * @param {string} successStatus 
 * @returns {FilteredResponse}
 */
export async function callResponse(method: string, url: string, body: any, successStatus: number): FilteredResponse {
    try {

        return {
            statusCode: successStatus,
            responseData: body   // list of the keys of the data object
        };

    } catch (err) {

        return {
            statusCode: 500,
            errorMessage: "Something went wrong processing this request. Please try again later or contact support if this error persists. Error Code: 5xU07",
            response: {
                method: method,
                url: url,
                successStatus: successStatus
            }
        };
    }
}