import { validateProps } from "../../src/types/utils";
import { UsedProperties, SegmentProperties, subConditionProperties } from "../../src/types/segments";
import { PropertiesDefault, PropertiesPossible } from "../../src/types/parameterValues";
import { segmentObjectProperties, parameterValuesObjectProperties } from "../../src/types/validation";
import { generateUuid, findAllIndicesOfCharacter, areArraysEqual, allSameType, getComparisonOperators } from "../utils/helpers";
import { sdkResponse } from "../../src/types/sdkResponse";

/**
 * @function validatePropertyName
 * @param {Array<any>} parameterValuesObject
 */
export function validatePropertyName(parameterValuesObject: PropertiesDefault): PropertiesDefault {

    // check if "properties." is not in front of string
    if (!parameterValuesObject.propertyName.startsWith("properties.")) {
        parameterValuesObject.propertyName = "properties." + parameterValuesObject.propertyName;
    }

    return parameterValuesObject;
}

export function formatPropertyName(subCondition: subConditionProperties): subConditionProperties {

    // check if "properties." is in front of string
    if (subCondition.parameterValues.propertyName.startsWith("properties.")) {
        subCondition.parameterValues.propertyName = subCondition.parameterValues.propertyName.replace("properties.", "");
    }

    return subCondition;
}

/**
 * @function validatePropertyValue
 * @param {PropertiesDefault} parameterValuesObject
 */
export function validatePropertyValue(parameterValuesObject: PropertiesDefault): parameterValuesObjectProperties {

    let isValidPropertyValueUsage = false;

    let validatedParameterValuesObject;

    let indicesDate = [4, 7]; // indices for the '-' character in yyyy-mm-dd

    let indices;

    switch (typeof (parameterValuesObject.propertyValue)) { // check type of given propertyValue
        case "number":

            validatedParameterValuesObject = {
                propertyName: parameterValuesObject.propertyName,
                comparisonOperator: parameterValuesObject.comparisonOperator,
                propertyValueInteger: parameterValuesObject.propertyValue
            };

            isValidPropertyValueUsage = true;

            break;

        case "string":

            indices = findAllIndicesOfCharacter(parameterValuesObject.propertyValue, "-");

            if (areArraysEqual(indicesDate, indices)) {

                validatedParameterValuesObject = {
                    propertyName: parameterValuesObject.propertyName,
                    comparisonOperator: parameterValuesObject.comparisonOperator,
                    propertyValueDate: parameterValuesObject.propertyValue
                };

                isValidPropertyValueUsage = true;
            }

            else {

                validatedParameterValuesObject = {
                    propertyName: parameterValuesObject.propertyName,
                    comparisonOperator: parameterValuesObject.comparisonOperator,
                    propertyValue: parameterValuesObject.propertyValue
                };

                isValidPropertyValueUsage = true;
            }

            break;

        case "object":

            // check if null or undefined, since null is treated as object by typeof
            if (parameterValuesObject.propertyValue == undefined || parameterValuesObject.propertyValue == null) {

                validatedParameterValuesObject = {
                    propertyName: parameterValuesObject.propertyName,
                    comparisonOperator: parameterValuesObject.comparisonOperator,
                };

                isValidPropertyValueUsage = true;
            }

            if (parameterValuesObject.propertyValue instanceof Array) { // check if array, since array is treated as object by typeof

                if (allSameType(parameterValuesObject.propertyValue)) { // check if all array valus are of the same type

                    switch (typeof (parameterValuesObject.propertyValue[0])) { // check type of first element in array (rest of elements should be same value!)
                        case "number":

                            validatedParameterValuesObject = {
                                propertyName: parameterValuesObject.propertyName,
                                comparisonOperator: parameterValuesObject.comparisonOperator,
                                propertyValuesInteger: parameterValuesObject.propertyValue
                            };

                            isValidPropertyValueUsage = true;

                            break;

                        case "string":

                            indices = findAllIndicesOfCharacter(parameterValuesObject.propertyValue[0], "-");

                            if (areArraysEqual(indicesDate, indices)) {

                                validatedParameterValuesObject = {
                                    propertyName: parameterValuesObject.propertyName,
                                    comparisonOperator: parameterValuesObject.comparisonOperator,
                                    propertyValuesDate: parameterValuesObject.propertyValue
                                };

                                isValidPropertyValueUsage = true;
                            }

                            else {

                                validatedParameterValuesObject = {
                                    propertyName: parameterValuesObject.propertyName,
                                    comparisonOperator: parameterValuesObject.comparisonOperator,
                                    propertyValues: parameterValuesObject.propertyValue
                                };

                                isValidPropertyValueUsage = true;
                            }

                            break;
                    }
                }
            }

            break;
    }

    return {
        isValidPropertyValueUsage: isValidPropertyValueUsage,
        validatedParameterValuesObject: validatedParameterValuesObject
    };
}

/**
 * @function validateComparisonOperatorUsage
 * @param {PropertiesPossible} parameterValuesObject
 */
export function validateComparisonOperatorUsage(parameterValuesObject: PropertiesPossible): boolean {

    let isValidComparisonOperatorUsage = false;

    const comparisonOperators = getComparisonOperators();

    const comparisonOperatorExists = comparisonOperators.includes(parameterValuesObject.comparisonOperator);

    if (comparisonOperatorExists) {
        const parameterValuesObjectKeys = Object.keys(parameterValuesObject);

        const propertyNameExists = parameterValuesObjectKeys.includes("propertyName");

        const propertyValueExists = parameterValuesObjectKeys.includes("propertyValue");
        const propertyValueIntegerExists = parameterValuesObjectKeys.includes("propertyValueInteger");
        const propertyValueDateExists = parameterValuesObjectKeys.includes("propertyValueDate");
        const propertyValuesExists = parameterValuesObjectKeys.includes("propertyValues");
        const propertyValuesIntegerExists = parameterValuesObjectKeys.includes("propertyValuesInteger");
        const propertyValuesDateExists = parameterValuesObjectKeys.includes("propertyValuesDate");

        switch (parameterValuesObject.comparisonOperator) {

            // these comparisonOperators do not accept arrays as propertyValue
            case "equals":
            case "notEquals":
            case "greaterThan":
            case "greaterThanOrEqualTo":
            case "lessThan":
            case "lessThanOrEqualTo":
                if (propertyValueExists || propertyValueIntegerExists || propertyValueDateExists) {
                    isValidComparisonOperatorUsage = true;
                }

                break;

            // these comparisonOperators only accept a single string as propertyValue
            case "contains":
            case "notContains":
            case "startsWith":
            case "endsWith":
                if (propertyValueExists) {
                    isValidComparisonOperatorUsage = true;
                }

                break;

            // these comparisonOperators do not need a propertyValue
            case "exists":
            case "missing":
                if (parameterValuesObjectKeys.length === 2 && comparisonOperatorExists && propertyNameExists) {
                    isValidComparisonOperatorUsage = true;
                }

                break;

            // these comparisonOperators do not accept single values as propertyValue
            case "between":
            case "in":
            case "notIn":
            case "all":
            case "hasSomeOf":
            case "hasNoneOf":
                if (propertyValuesExists || propertyValuesIntegerExists || propertyValuesDateExists) {
                    isValidComparisonOperatorUsage = true;
                }

                break;
        }
    }

    return isValidComparisonOperatorUsage;
}

/**
 * @function validateSegmentObject
 * @param {UsedProperties} params
 * @param {boolean} segmentUpdate
 */
export function validateSegmentObject(params: UsedProperties, segmentUpdate: boolean): segmentObjectProperties {
    let subConditions = new Array;

    let id;

    let errorCode

    let areSubConditionsValid = true;

    for (let index = 0; index < params.subConditions.length; index++) {

        const subCondition = params.subConditions[index];

        const validationPropertyNameResponse = validatePropertyName(subCondition);

        const validationPropertyValueResponse = validatePropertyValue(validationPropertyNameResponse);

        if (validationPropertyValueResponse.isValidPropertyValueUsage) {

            const isValidComparisonOperatorUsage = validateComparisonOperatorUsage(validationPropertyValueResponse.validatedParameterValuesObject);

            // used propertyValue(s) and comparisonOperator combination is not valid
            if (!isValidComparisonOperatorUsage) {

                areSubConditionsValid = false;

                errorCode = "5xU03";
            }
        }

        else { // used propertyValue(s) is not valid

            areSubConditionsValid = false;

            errorCode = "5xU02";
        }

        subConditions.push({
            type: "profilePropertyCondition",
            parameterValues: validationPropertyValueResponse.validatedParameterValuesObject
        });
    };

    // if not valid subconditions, then return the user's original subconditions so the mistake(s) can be seen and corrected
    if (!areSubConditionsValid) subConditions = params.subConditions;

    // if segment update function
    if (segmentUpdate) id = params.id;
    else id = generateUuid();

    const segmentObject = {
        metadata: {
            id: id,
            name: params.name,
            description: params.description,
            scope: params.scope,
            tags: params.tags,
            systemTags: params.systemTags
        },
        condition: {
            type: "booleanCondition",
            parameterValues: {
                operator: params.operator,
                subConditions: subConditions
            }
        }
    };

    const validatedSegmentObject = {
        segmentObject: segmentObject,
        errorCode: errorCode,
        areSubConditionsValid: areSubConditionsValid
    }

    return validatedSegmentObject;
}

/**
 * @function validateRequiredProps
 * @param {string[]} required
 * @param {{[key: string]: any}} props
 */
export function validateRequiredProps(required: string[], props: { [key: string]: any }): validateProps {

    let missing = [];

    for (const prop of required) {

        if (!Object.keys(props).includes(prop) || props[prop] === null || props[prop] === undefined) {

            missing.push(prop);
        }
    };

    return {
        valid: !missing.length,
        missing,
    };
}

export function formatSubCondition(subCondition: subConditionProperties): any {

    const parameterValuesObjectKeys = Object.keys(subCondition.parameterValues);

    const propertyValueExists = parameterValuesObjectKeys.includes("propertyValue");
    const propertyValueIntegerExists = parameterValuesObjectKeys.includes("propertyValueInteger");
    const propertyValueDateExists = parameterValuesObjectKeys.includes("propertyValueDate");
    const propertyValuesExists = parameterValuesObjectKeys.includes("propertyValues");
    const propertyValuesIntegerExists = parameterValuesObjectKeys.includes("propertyValuesInteger");
    const propertyValuesDateExists = parameterValuesObjectKeys.includes("propertyValuesDate");

    let formattedSubCondition = {
        propertyName: subCondition.parameterValues.propertyName,
        comparisonOperator: subCondition.parameterValues.comparisonOperator
    };

    if (propertyValueExists) {
        Object.assign(formattedSubCondition, { propertyValue: subCondition.parameterValues.propertyValue });
    }

    else if (propertyValueIntegerExists) {
        Object.assign(formattedSubCondition, { propertyValue: subCondition.parameterValues.propertyValueInteger });
    }

    else if (propertyValueDateExists) {
        Object.assign(formattedSubCondition, { propertyValue: subCondition.parameterValues.propertyValueDate });
    }

    else if (propertyValuesExists) {
        Object.assign(formattedSubCondition, { propertyValue: subCondition.parameterValues.propertyValues });
    }

    else if (propertyValuesIntegerExists) {
        Object.assign(formattedSubCondition, { propertyValue: subCondition.parameterValues.propertyValuesInteger });
    }

    else if (propertyValuesDateExists) {
        Object.assign(formattedSubCondition, { propertyValue: subCondition.parameterValues.propertyValuesDate });
    }

    else {
        Object.assign(formattedSubCondition, { propertyValue: null });
    }

    return formattedSubCondition
}

export function reformatSegment(sdkObject: sdkResponse, segment: SegmentProperties): sdkResponse {

    let formattedSubConditions = new Array;

    for (let index = 0; index < segment.condition.parameterValues.subConditions.length; index++) {
        let subCondition = formatPropertyName(segment.condition.parameterValues.subConditions[index]);

        formattedSubConditions.push(formatSubCondition(subCondition));
    }

    const segmentObject = {
        id: segment.metadata.id,
        name: segment.metadata.name,
        description: segment.metadata.description,
        scope: segment.metadata.scope,
        tags: segment.metadata.tags,
        systemTags: segment.metadata.systemTags,
        operator: segment.condition.parameterValues.operator,
        subConditions: formattedSubConditions
    };

    sdkObject.responseData = segmentObject;

    return sdkObject;
}